@@ -16,10 +16,10 @@
 #define CG_REF_HPP
 
 #include "SparseMatrix.hpp"
-#include "Geometry.hpp"
+#include "Vector.hpp"
 #include "CGData.hpp"
 
-int CG_ref(const Geometry & geom, const SparseMatrix & A, CGData & data, const double * const b, double * const x,
+int CG_ref(const SparseMatrix & A, CGData & data, const Vector & b, Vector & x,
     const int max_iter, const double tolerance, int & niters, double & normr,  double & normr0,
     double * times, bool doPreconditioning);
 
