@@ -15,7 +15,8 @@
 #ifndef COMPUTESYMGS_HPP
 #define COMPUTESYMGS_HPP
 #include "SparseMatrix.hpp"
+#include "Vector.hpp"
 
-int ComputeSYMGS( const SparseMatrix  & A, const double * const x, double * const y);
+int ComputeSYMGS( const SparseMatrix  & A, const Vector & x, Vector & y);
 
 #endif // COMPUTESYMGS_HPP
