@@ -23,8 +23,7 @@
   Optimizes the data structures used for CG iteration to increase the
   performance of the benchmark version of the preconditioned CG algorithm.
 
-  @param[in]    geom   The description of the problem's geometry.
-  @param[inout] A      The known system matrix
+  @param[inout] A      The known system matrix, also contains the MG hierarchy in attributes Ac and mgData.
   @param[inout] data   The data structure with all necessary CG vectors preallocated
   @param[inout] b      The known right hand side vector
   @param[inout] x      The solution vector to be computed in future CG iteration
@@ -35,7 +34,7 @@
   @see GenerateGeometry
   @see GenerateProblem
 */
-int OptimizeProblem(const Geometry & geom, SparseMatrix & A, CGData & data, double * b, double * x, double * xexact) {
+int OptimizeProblem(SparseMatrix & A, CGData & data, Vector & b, Vector & x, Vector & xexact) {
 
 // This function can be used to completely transform any part of the data structures.
 // Right now it does nothing, so compiling with a check for unused variables results in complaints
