@@ -19,7 +19,7 @@
 #include "TestSymmetry.hpp"
 #include "TestNorms.hpp"
 
-void ReportResults(const SparseMatrix & A, int numberOfMgLevels, int numberOfCgSets, int refMaxIters, int niters, double times[],
+void ReportResults(const SparseMatrix & A, int numberOfMgLevels, int numberOfCgSets, int refMaxIters, int optMaxIters, double times[],
     const TestCGData & testcg_data, const TestSymmetryData & testsymmetry_data, const TestNormsData & testnorms_data, int global_failure);
 
 #endif // REPORTRESULTS_HPP
