@@ -23,10 +23,14 @@
 
 #include <map>
 #include <vector>
+#include <cassert>
 #include "Geometry.hpp"
+#include "Vector.hpp"
+#include "MGData.hpp"
 
 struct SparseMatrix_STRUCT {
   char  * title; //!< name of the sparse matrix
+  Geometry * geom; //!< geometry associated with this matrix
   global_int_t totalNumberOfRows; //!< total number of matrix rows across all processes
   global_int_t totalNumberOfNonzeros; //!< total number of matrix nonzeros across all processes
   local_int_t localNumberOfRows; //!< number of rows local to this process
@@ -41,13 +45,15 @@ struct SparseMatrix_STRUCT {
   std::vector< global_int_t > localToGlobalMap; //!< local-to-global mapping
   mutable bool isDotProductOptimized;
   mutable bool isSpmvOptimized;
-  mutable bool isSymgsOptimized;
+  mutable bool isMgOptimized;
   mutable bool isWaxpbyOptimized;
   /*!
    This is for storing optimized data structres created in OptimizeProblem and
    used inside optimized ComputeSPMV().
    */
-  void * optimization_data;
+  mutable struct SparseMatrix_STRUCT * Ac; // Coarse grid matrix
+  mutable MGData * mgData; // Pointer to the coarse level data for this fine matrix
+  void * optimizationData;  // pointer that can be used to store implementation-specific data
 
 #ifndef HPCG_NOMPI
   local_int_t numberOfExternalValues; //!< number of entries that are external to this process
@@ -67,8 +73,9 @@ typedef struct SparseMatrix_STRUCT SparseMatrix;
 
   @param[in] A the known system matrix
  */
-inline void InitializeSparseMatrix(SparseMatrix & A) {
+inline void InitializeSparseMatrix(SparseMatrix & A, Geometry * geom) {
   A.title = 0;
+  A.geom = geom;
   A.totalNumberOfRows = 0;
   A.totalNumberOfNonzeros = 0;
   A.localNumberOfRows = 0;
@@ -84,7 +91,7 @@ inline void InitializeSparseMatrix(SparseMatrix & A) {
   // functions that are meant to be optimized.
   A.isDotProductOptimized = true;
   A.isSpmvOptimized       = true;
-  A.isSymgsOptimized      = true;
+  A.isMgOptimized      = true;
   A.isWaxpbyOptimized     = true;
 
 #ifndef HPCG_NOMPI
@@ -97,9 +104,37 @@ inline void InitializeSparseMatrix(SparseMatrix & A) {
   A.sendLength = 0;
   A.sendBuffer = 0;
 #endif
+  A.mgData = 0; // Fine-to-coarse grid transfer initially not defined.
+  A.Ac =0;
   return;
 }
 
+/*!
+  Copy values from matrix diagonal into user-provided vector.
+
+  @param[in] A the known system matrix.
+  @param[inout] diagonal  Vector of diagonal values (must be allocated before call to this function).
+ */
+inline void CopyMatrixDiagonal(SparseMatrix & A, Vector & diagonal) {
+    double ** curDiagA = A.matrixDiagonal;
+    double * dv = diagonal.values;
+    assert(A.localNumberOfRows==diagonal.localLength);
+    for (local_int_t i=0; i<A.localNumberOfRows; ++i) dv[i] = *(curDiagA[i]);
+  return;
+}
+/*!
+  Replace specified matrix diagonal value.
+
+  @param[inout] A The system matrix.
+  @param[in] diagonal  Vector of diagonal values that will replace existing matrix diagonal values.
+ */
+inline void ReplaceMatrixDiagonal(SparseMatrix & A, Vector & diagonal) {
+    double ** curDiagA = A.matrixDiagonal;
+    double * dv = diagonal.values;
+    assert(A.localNumberOfRows==diagonal.localLength);
+    for (local_int_t i=0; i<A.localNumberOfRows; ++i) *(curDiagA[i]) = dv[i];
+  return;
+}
 /*!
   Deallocates the members of the data structure of the known system matrix provided they are not 0.
 
@@ -112,6 +147,7 @@ inline void DeleteMatrix(SparseMatrix & A) {
     delete [] A.mtxIndG[i];
     delete [] A.mtxIndL[i];
   }
+
   if (A.title)                  delete [] A.title;
   if (A.nonzerosInRow)             delete [] A.nonzerosInRow;
   if (A.mtxIndG) delete [] A.mtxIndG;
@@ -126,7 +162,10 @@ inline void DeleteMatrix(SparseMatrix & A) {
   if (A.sendLength)            delete [] A.sendLength;
   if (A.sendBuffer)            delete [] A.sendBuffer;
 #endif
-  InitializeSparseMatrix(A);
+
+  if (A.geom!=0) { delete A.geom; A.geom = 0;}
+  if (A.Ac!=0) { DeleteMatrix(*A.Ac); delete A.Ac; A.Ac = 0;} // Delete coarse matrix
+  if (A.mgData!=0) { DeleteMGData(*A.mgData); delete A.mgData; A.mgData = 0;} // Delete MG data
   return;
 }
 
